/*
 * CFGCPROPAGATOR.h
 *
 *  Created on: Sep 17, 2012
 *      Author: jun
 */

#ifndef CFGCPROPAGATOR_H_
#define CFGCPROPAGATOR_H_

#include <gecode/int.hh>
#include "parse-graph.h"

class CFGC : public Propagator {

protected:
  // advisor
  class ViewAdvisor : public Advisor {
  public:
    Int::IntView x;
    int var_ind;

    ViewAdvisor(Space& home, Propagator& p,
                Council<ViewAdvisor>& c, Int::IntView x0, int ind)
      : Advisor(home,p,c), x(x0), var_ind(ind)
    {
    	x.subscribe(home,*this);
    }

    ViewAdvisor(Space& home, bool share, ViewAdvisor& a)
      : Advisor(home,share,a)
    {
    	x.update(home,share,a.x);
    	var_ind = a.var_ind;
    }

    void dispose(Space& home, Council<ViewAdvisor>& c)
    {
    	x.cancel(home,*this);
    	Advisor::dispose(home,c);
    }

  };

public:

  ViewArray<Int::IntView> variables;
  Council<ViewAdvisor> c;
  int n;
  ParseGraph* parse_graph_pr;
  int cst_id;
  set<int> chg_pos;
  static int prop_count;
  int prop_id;

  // constructor for posting
  CFGC(Home home, ViewArray<Int::IntView>& x, ParseGraph* t_parse_graph_pr, int id)
    : Propagator(home), variables(x), c(home), n(x.size()),
      parse_graph_pr(t_parse_graph_pr), cst_id(id)
  {
	  for (int i=x.size(); i--; )
	  {
		  (void) new (home) ViewAdvisor(home,*this,c,x[i],i);
	  }

	  home.notice(*this,AP_DISPOSE);

	  prop_count++;
	  prop_id = prop_count;
	  parse_graph_pr->prop_id = prop_id;
	  chg_pos.clear();

	  parse_graph_pr->push_dummy_node(prop_id);
  }

  static ExecStatus post(Home home, ViewArray<Int::IntView>& x, ParseGraph* t_parse_graph_pr,
		  int id)
  {
	  (void) new (home) CFGC(home, x, t_parse_graph_pr, id);
	  return ES_OK;
  }

  // disposal
  virtual size_t dispose(Space& home)
  {
	  home.ignore(*this,AP_DISPOSE);
	  c.dispose(home);
	  (void) Propagator::dispose(home);

	  return sizeof(*this);
  }

  CFGC(Space& home, bool share, CFGC& p)
    : Propagator(home,share,p)
  {
	  c.update(home,share,p.c);
	  variables.update(home, share, p.variables);
	  n = p.n;
	  parse_graph_pr = p.parse_graph_pr;
	  cst_id = p.cst_id;

	  prop_count++;
	  prop_id = prop_count;

	  chg_pos.clear();
  }

  virtual Propagator* copy(Space& home, bool share)
  {
	  Propagator* pr = new (home) CFGC(home,share,*this);

	  if(parse_graph_pr->prop_id==prop_id)
	  {
		  parse_graph_pr->push_dummy_node(prop_count);
	  }
	  else
	  {
		  parse_graph_pr->pop_back_up_nodes(prop_id+1);
	  }

/*	  cout << "cst_id is " << cst_id << ", propagator " << pr << " id " << prop_count
	       << " is copied from " << this << " id " << prop_id << endl;*/

	  return pr;
  }

  virtual PropCost cost(const Space&, const ModEventDelta&) const
  {
	  return PropCost::cubic(PropCost::HI,n);
  }

  // advise function
  virtual ExecStatus advise(Space&, Advisor& a, const Delta&)
  {
	  ViewAdvisor& cur_adv = static_cast<ViewAdvisor&>(a);
	  chg_pos.insert(cur_adv.var_ind);

	  return ES_NOFIX;
  }

  // propagation
  virtual ExecStatus propagate(Space& home, const ModEventDelta&)
  {
	  if(prop_id!=parse_graph_pr->prop_id)
	  {
		  parse_graph_pr->prop_id = prop_id;

//		  cout << "cst_id is " << cst_id << " backtrack to propagator id " << prop_id << endl;

		  parse_graph_pr->pop_back_up_nodes(prop_id);
		  parse_graph_pr->push_dummy_node(prop_id);

		  for(int i=0; i<n; i++)
			  chg_pos.insert(i);
	  }

	  ExecStatus exe_status = parse_graph_pr->filtering_from_update(home, chg_pos, variables);

	  chg_pos.clear();

	  return exe_status;
  }

};

int CFGC::prop_count = 0;

void cfgc(Home home, const IntVarArgs& x, ParseGraph* parse_graph_pr, int id)
{
	if (home.failed()) return;

	ViewArray<Int::IntView> y(home, x);

	GECODE_ES_FAIL(parse_graph_pr->build_parse_graph(home, y, id));

	GECODE_ES_FAIL(CFGC::post(home, y, parse_graph_pr, id));
}




#endif /* CFGCPROPAGATOR_H_ */
