/*
 * cfg.h
 *
 *  Created on: Aug 13, 2012
 *      Author: jun
 */

#ifndef CFG_H_
#define CFG_H_

#include <stdio.h>
#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <map>
#include <set>
#include <sstream>
#include <algorithm>
#include <assert.h>
#include <stack>
#include <queue>

using namespace std;

bool read_char_vals(string fileName, map<char,int>& char_to_word,
		map<int,char>& word_name, int& word_count)
{
	char *filePath = (char*)fileName.c_str();
	ifstream cfgFile(filePath);

	if(!cfgFile.is_open())  return false;

	for(string readLine; getline(cfgFile,readLine); )
	{
		int pos = readLine.find('"');
		while(pos>=0)
		{
			readLine = readLine.substr(pos+1);

			assert(readLine[1]=='"');

			char cur_char = readLine[0];

			map<char,int>::iterator char_to_word_it
			    = char_to_word.find(cur_char);
			if(char_to_word_it==char_to_word.end())
			{
				word_count++;
				word_name[word_count] = cur_char;
				char_to_word[cur_char] = word_count;
			}

			readLine = readLine.substr(2);
			pos = readLine.find('"');
		}
	}

	cfgFile.close();

	return true;
}

string trim(string in_str)
{
	in_str.erase(0, in_str.find_first_not_of(" "));
	in_str.erase(in_str.find_last_not_of(" ") + 1);
	return in_str;
}

string trimFirst(string in_str)
{
	in_str.erase(0, in_str.find_first_not_of(" "));
	return in_str;
}

struct Element
{
	int type; // 1 for symbol, 2 for a char value
	int value;
};

bool operator <(const Element& ele1, const Element& ele2)
{
	if(ele1.type<ele2.type) return true;
	if(ele1.type>ele2.type) return false;

	if(ele1.value<ele2.value) return true;
	else return false;
}

bool operator ==(const Element& ele1, const Element& ele2)
{
	if(ele1.type!=ele2.type) return false;
	if(ele1.value!=ele2.value) return false;

	return true;
}

bool operator >(const Element& ele1, const Element& ele2)
{
	if(ele1.type<ele2.type) return false;
	if(ele1.type>ele2.type) return true;

	if(ele1.value>ele2.value) return true;
	else return false;
}

bool operator !=(const Element& ele1, const Element& ele2)
{
	if(ele1.type!=ele2.type) return true;
	if(ele1.value!=ele2.value) return true;

	return false;
}

bool operator <(const vector<Element>& ele_vec1, const vector<Element>& ele_vec2)
{
	int size1 = ele_vec1.size();
	int size2 = ele_vec2.size();

	if(size1>size2) return false;
	else if(size1<size2) return true;
	else
	{
		for(int i=0; i<size1; i++)
		{
			Element ele1 = ele_vec1[i];
			Element ele2 = ele_vec2[i];

			if(ele1<ele2) return true;
			else if(ele1>ele2) return false;
		}
	}
    return false;
}

bool operator ==(const vector<Element>& ele_vec1, const vector<Element>& ele_vec2)
{
	int size1 = ele_vec1.size();
	int size2 = ele_vec2.size();

	if(size1!=size2) return false;
	else
	{
		for(int i=0; i<size1; i++)
		{
			Element ele1 = ele_vec1[i];
			Element ele2 = ele_vec2[i];

			if(ele1!=ele2) return false;
		}
	}
    return true;
}

bool operator >(const vector<Element>& ele_vec1, const vector<Element>& ele_vec2)
{
	int size1 = ele_vec1.size();
	int size2 = ele_vec2.size();

	if(size1>size2) return true;
	else if(size1<size2) return false;
	else
	{
		for(int i=0; i<size1; i++)
		{
			Element ele1 = ele_vec1[i];
			Element ele2 = ele_vec2[i];

			if(ele1<ele2) return false;
			else if(ele1>ele2) return true;
		}
	}
    return false;
}

struct CNFProduction
{
	int gen_sym;
	int from_first;
	int from_second;

	CNFProduction(int t_gen_sym, int t_from_first, int t_from_second)
	:gen_sym(t_gen_sym), from_first(t_from_first), from_second(t_from_second){};
};

struct Production
{
	int genSym;
	vector<Element> ele_vec;
};

struct ProdList
{
	Production* prod_pr;
	ProdList* next;
};

struct SymPosition
{
	Production* prod;
	int ele_pos;
};

struct SymPosList
{
	SymPosition* sym_pos_pr;
	SymPosList* next;
};

struct CNFProdList
{
	CNFProduction* CNFProd_pr;
	CNFProdList* next;

	CNFProdList(CNFProduction* t_CNFProd_pr)
	:CNFProd_pr(t_CNFProd_pr), next(NULL){};
};

struct CNFProdListPr
{
	CNFProdList* head_pr;
	CNFProdList* end_pr;

	CNFProdListPr()
	{
		head_pr = NULL;
		end_pr = NULL;
	}
};

class CFG
{
public:
	CFG(string fileName, const map<string,int>& t_string_to_nonterminal,
			const map<int,string>& t_nonterminal_name, const int& t_terminal_count);
	~CFG();
	void print();
	void addSymPosition(int t_sym, Production* t_prod, int t_ele_pos);
	void removeEmpson();
	void removeUnitProduction();
	void removeUselessSym();
	void makeCNF();
	void printCNF();

	int start_sym;

	CNFProdListPr cnf_prod_pr;
	vector<CNFProdListPr> left_child_sym_to_cnf_prod;

	vector<vector<int> > terminal_to_head_sym_vec;
	map<int,set<int> > head_sym_to_terminal_set;
	set<int> sym_gen_from_char;

	map<int,string> sym_name;
	map<string,int> str_to_sym;
	map<string,int>::iterator str_to_sym_it;

	map<int,string> terminal_name;
	map<string,int> string_to_terminal;
	map<string,int>::iterator string_to_terminal_it;

	ProdList* prod_list_head;
	ProdList* prod_list_end;

	vector<Element>::iterator ele_it;

	int sym_count;
	int terminal_count;

	map<int,SymPosList*> sym_to_pos_head;
	map<int,SymPosList*>::iterator sym_to_pos_head_it;
	map<int,SymPosList*> sym_to_pos_end;

	set<int> emp_sym;
	map<int,set<vector<Element> > > sym_to_gen_ele;
	map<int,set<vector<Element> > >::iterator sym_to_gen_ele_it;
	set<vector<Element> >::iterator ele_set_it;

	map<int,ProdList*> gen_sym_to_prod_head;
	map<int,ProdList*> gen_sym_to_prod_end;
	map<int,ProdList*>::iterator gen_sym_to_prod_it;

	set<int> generating_sym;
	set<int> reachable_sym;
	set<int> reachable_terminal;
	set<int> useful_sym;
	set<int> useful_terminal;

	bool input_file_error;
};

CFG::CFG(string fileName, const map<string,int>& t_string_to_nonterminal,
		const map<int,string>& t_nonterminal_name, const int& t_terminal_count)
{
	start_sym = 1;

	input_file_error = false;

	sym_name.clear();
	str_to_sym.clear();

	sym_to_pos_head.clear();
	sym_to_pos_end.clear();
	sym_count = 0;

	generating_sym.clear();
	reachable_sym.clear();
	reachable_terminal.clear();
	useful_sym.clear();
	useful_terminal.clear();

	string_to_terminal = t_string_to_nonterminal;
	terminal_name = t_nonterminal_name;
	terminal_count = t_terminal_count;

	prod_list_head = new ProdList();
	prod_list_head->next = NULL;
	prod_list_head->prod_pr = NULL;
	prod_list_end = prod_list_head;

	emp_sym.clear();

	char *filePath = (char*)fileName.c_str();
	ifstream cfgFile(filePath);

	if(!cfgFile.is_open())
	{
		input_file_error = true;
		cout << "Error: The input grammar file is not existed!" << endl;
		return;
	}

	for(string readLine; getline(cfgFile,readLine); )
	{
		readLine = trimFirst(readLine);
		int pos = readLine.find("cfg ");
		if(pos==0)
		{
			Production* new_prod = new Production();
			new_prod->ele_vec.clear();
			new_prod->ele_vec.reserve(5);

			if(prod_list_end->prod_pr==NULL)
			{
				prod_list_end->prod_pr = new_prod;
			}
			else
			{
				prod_list_end->next = new ProdList();
				prod_list_end = prod_list_end->next;
				prod_list_end->prod_pr = new_prod;
				prod_list_end->next = NULL;
			}

			int cur_genSym;
			int cur_terminal;
			int cur_symbol;

			readLine = readLine.substr(4);
			readLine = trimFirst(readLine);

			pos = readLine.find(":=");
			if(pos!=string::npos)
			{
				string sub_head = readLine.substr(0,pos);
				readLine = readLine.substr(pos+2);
				sub_head = trim(sub_head);

				str_to_sym_it = str_to_sym.find(sub_head);
				if(str_to_sym_it==str_to_sym.end())
				{
					sym_count++;
					sym_name[sym_count] = sub_head;
					str_to_sym[sub_head] = sym_count;
					cur_genSym = sym_count;
				}
				else cur_genSym = str_to_sym_it->second;

				new_prod->genSym = cur_genSym;

				sym_to_gen_ele_it = sym_to_gen_ele.find(cur_genSym);
				if(sym_to_gen_ele_it==sym_to_gen_ele.end())
				{
					set<vector<Element> > vec_set;
					vec_set.clear();
					sym_to_gen_ele[cur_genSym] = vec_set;
				}

				readLine = trimFirst(readLine);
				while(readLine[0]!=';')
				{
					if(readLine[0]=='|')  // generating a new production
					{
						sym_to_gen_ele[cur_genSym].insert(new_prod->ele_vec);

						if(new_prod->ele_vec.size()==0)
						{
							emp_sym.insert(cur_genSym);
						}

						new_prod = new Production();
						new_prod->genSym = cur_genSym;
						new_prod->ele_vec.clear();
						new_prod->ele_vec.reserve(5);

						prod_list_end->next = new ProdList();
						prod_list_end = prod_list_end->next;
						prod_list_end->prod_pr = new_prod;
						prod_list_end->next = NULL;

						readLine = readLine.substr(1);
					}
					else if(readLine[0]=='"')  // reading a char value
					{
						readLine = readLine.substr(1);

						pos = readLine.find("\"");

						if(pos==string::npos)
						{
							input_file_error = true;
							cout << "Error: the cfg file is wrong, as the char value is illegal!"
									<< endl;
							return;
						}
						else
						{
							Element new_ele;
							new_ele.type = 2;

							string cur_ter_string = readLine.substr(0,pos);

							string_to_terminal_it =
									string_to_terminal.find(cur_ter_string);
							assert(string_to_terminal_it!=string_to_terminal.end());

							cur_terminal = string_to_terminal_it->second;

							new_ele.value = cur_terminal;
							new_prod->ele_vec.push_back(new_ele);

							readLine = readLine.substr(pos+1);
						}
					}
					else  // reading a symbol
					{
						pos = readLine.find(" ");
						if(pos!=string::npos)
						{
							sub_head = readLine.substr(0,pos);
							readLine = readLine.substr(pos+1);

							str_to_sym_it = str_to_sym.find(sub_head);
							if(str_to_sym_it==str_to_sym.end())
							{
								sym_count++;
								sym_name[sym_count] = sub_head;
								str_to_sym[sub_head] = sym_count;
								cur_symbol = sym_count;
							}
							else cur_symbol = str_to_sym_it->second;

							addSymPosition(cur_symbol, new_prod, new_prod->ele_vec.size());

							Element new_ele;
							new_ele.type = 1;
                            new_ele.value = cur_symbol;
							new_prod->ele_vec.push_back(new_ele);
						}
						else
						{
							cout << "Error: the cfg file is wrong, as no space after a symbol!"
									<< endl;
							input_file_error = true;
							return;
						}
					}

					readLine = trimFirst(readLine);
			    }

				sym_to_gen_ele[cur_genSym].insert(new_prod->ele_vec);

				if(new_prod->ele_vec.size()==0)
				{
					emp_sym.insert(cur_genSym);
				}
			}
			else
			{
				cout << "Error: the cfg file is wrong, as no cfg head is given!" << endl;
				input_file_error = true;
				return;
			}
		}
		else
		{
			cout << "Error: the cfg file is wrong, as no cfg keyword is found!" << endl;
			input_file_error = true;
			return;
		}
	}

	cfgFile.close();
}

void CFG::removeEmpson()
{
	set<int>::iterator emp_sym_it;
	set<int> old_emp_sym;
	old_emp_sym.clear();

	while(!emp_sym.empty())
	{
		emp_sym_it = emp_sym.begin();
		int cur_emp_sym = *emp_sym_it;
		emp_sym.erase(emp_sym_it);
		old_emp_sym.insert(cur_emp_sym);

		sym_to_pos_head_it = sym_to_pos_head.find(cur_emp_sym);
		if(sym_to_pos_head_it!=sym_to_pos_head.end())
		{
			SymPosList* sym_to_pos_cur = sym_to_pos_head[cur_emp_sym];
			while(sym_to_pos_cur!=NULL)
			{
				Production* cur_prod = sym_to_pos_cur->sym_pos_pr->prod;
				int cur_pos = sym_to_pos_cur->sym_pos_pr->ele_pos;
				int cur_gen_sym = cur_prod->genSym;

				int ele_vec_size = cur_prod->ele_vec.size();
				if(ele_vec_size==1)
				{
					emp_sym_it = old_emp_sym.find(cur_gen_sym);
					if(emp_sym_it==old_emp_sym.end())
					{
						emp_sym.insert(cur_gen_sym);
					}
				}
				else
				{
					vector<Element> new_ele_vec;
					new_ele_vec.clear();
					new_ele_vec.reserve(5);

					for(int i=0; i<ele_vec_size; i++)
					{
						if(i!=cur_pos)
						{
							new_ele_vec.push_back(cur_prod->ele_vec[i]);
						}

					}

					ele_set_it = sym_to_gen_ele[cur_gen_sym].find(new_ele_vec);
					if(ele_set_it==sym_to_gen_ele[cur_gen_sym].end())
					{
						sym_to_gen_ele[cur_gen_sym].insert(new_ele_vec);
						Production* new_prod = new Production;
						new_prod->ele_vec = new_ele_vec;
						new_prod->genSym = cur_gen_sym;

						prod_list_end->next = new ProdList();
						prod_list_end = prod_list_end->next;
						prod_list_end->prod_pr = new_prod;
						prod_list_end->next = NULL;

						ele_vec_size--;
						for(int j=0; j<ele_vec_size; j++)
						{
							Element cur_ele = new_ele_vec[j];
							if(cur_ele.type==1)
							{
								addSymPosition(cur_ele.value, new_prod, j);
							}
						}

					}

				}

				sym_to_pos_cur = sym_to_pos_cur->next;
			}
		}
	}

	////------------  free unused memory ------------------------////
	for(sym_to_pos_head_it = sym_to_pos_head.begin();
	       sym_to_pos_head_it != sym_to_pos_head.end(); sym_to_pos_head_it++)
	{
		SymPosList* t_sym_pos_list = sym_to_pos_head_it->second;
		while(t_sym_pos_list!=NULL)
		{
			delete t_sym_pos_list->sym_pos_pr;
			SymPosList* temp_sym_pos_list = t_sym_pos_list;
			t_sym_pos_list = t_sym_pos_list->next;
			delete temp_sym_pos_list;
		}
	}
}

void CFG::removeUnitProduction()
{
	map<int,set<int> > link_from;
	link_from.clear();
	map<int,set<int> > link_to;
	link_to.clear();
	set<int>::iterator set_int_it;
	map<int,set<int> > ::iterator int_to_set_int_it;
	set<int> col_has_val;
	col_has_val.clear();

	ProdList* prod_list_cur = prod_list_head;
	ProdList* prod_list_prev = NULL;
	while(prod_list_cur!=NULL)
	{
		Production* cur_prod = prod_list_cur->prod_pr;
		if(cur_prod==NULL)
		{
			prod_list_cur = prod_list_cur->next;
			continue;
		}

		vector<Element> cur_ele_vec = cur_prod->ele_vec;
		int cur_ele_vec_size = cur_ele_vec.size();
		int cur_gen_sym = cur_prod->genSym;

		if(cur_ele_vec_size==0)  // empson productions
		{
			delete cur_prod;

			if(prod_list_cur==prod_list_head)
			{
				prod_list_head = prod_list_cur->next;
				delete prod_list_cur;
				prod_list_cur = prod_list_head;
			}
			else
			{
				prod_list_prev->next = prod_list_cur->next;
				delete prod_list_cur;
				prod_list_cur = prod_list_prev->next;
			}
		}
		else if(cur_ele_vec_size==1) // find all unit productions
		{
			if(cur_ele_vec[0].type==1)  // an unit production
			{
				int t_from = cur_gen_sym;
				int t_to = cur_ele_vec[0].value;
				col_has_val.insert(t_to);

				int_to_set_int_it = link_from.find(t_to);
				if(int_to_set_int_it == link_from.end())
				{
					set<int> temp_s;
					temp_s.clear();
					link_from[t_to] = temp_s;

				}
				link_from[t_to].insert(t_from);

				int_to_set_int_it = link_to.find(t_from);
				if(int_to_set_int_it == link_to.end())
				{
					set<int> temp_s;
					temp_s.clear();
					link_to[t_from] = temp_s;

				}
				link_to[t_from].insert(t_to);

				delete cur_prod;

				if(prod_list_cur==prod_list_head)
				{
					prod_list_head = prod_list_cur->next;
					delete prod_list_cur;
					prod_list_cur = prod_list_head;
				}
				else
				{
					prod_list_prev->next = prod_list_cur->next;
					delete prod_list_cur;
					prod_list_cur = prod_list_prev->next;
				}
			}
			else   // not an unit production from a char value
			{
				generating_sym.insert(cur_gen_sym);

				gen_sym_to_prod_it = gen_sym_to_prod_head.find(cur_gen_sym);
				if(gen_sym_to_prod_it==gen_sym_to_prod_head.end())
				{
					gen_sym_to_prod_head[cur_gen_sym] = new ProdList;
					gen_sym_to_prod_end[cur_gen_sym] = gen_sym_to_prod_head[cur_gen_sym];
					gen_sym_to_prod_end[cur_gen_sym]->next = NULL;
					gen_sym_to_prod_end[cur_gen_sym]->prod_pr = cur_prod;
				}
				else
				{
					gen_sym_to_prod_end[cur_gen_sym]->next = new ProdList;
					gen_sym_to_prod_end[cur_gen_sym] = gen_sym_to_prod_end[cur_gen_sym]->next;
					gen_sym_to_prod_end[cur_gen_sym]->next = NULL;
					gen_sym_to_prod_end[cur_gen_sym]->prod_pr = cur_prod;
				}

				prod_list_prev = prod_list_cur;
				prod_list_cur = prod_list_cur->next;
			}
		}
		else  //  not an unit production from more than one char or symbol
		{
			gen_sym_to_prod_it = gen_sym_to_prod_head.find(cur_gen_sym);
			if(gen_sym_to_prod_it==gen_sym_to_prod_head.end())
			{
				gen_sym_to_prod_head[cur_gen_sym] = new ProdList;
				gen_sym_to_prod_end[cur_gen_sym] = gen_sym_to_prod_head[cur_gen_sym];
				gen_sym_to_prod_end[cur_gen_sym]->next = NULL;
				gen_sym_to_prod_end[cur_gen_sym]->prod_pr = cur_prod;
			}
			else
			{
				gen_sym_to_prod_end[cur_gen_sym]->next = new ProdList;
				gen_sym_to_prod_end[cur_gen_sym] = gen_sym_to_prod_end[cur_gen_sym]->next;
				gen_sym_to_prod_end[cur_gen_sym]->next = NULL;
				gen_sym_to_prod_end[cur_gen_sym]->prod_pr = cur_prod;
			}

			prod_list_prev = prod_list_cur;
			prod_list_cur = prod_list_cur->next;
		}
	}

	prod_list_end = prod_list_prev;

	///--------- implement the Warshall algorithm ------------------//
	///--------- http://class.htu.cn/lisanshuxue/neirong/3_8.htm  -----//
	set<int>::iterator temp_set_int_it;
	while(!col_has_val.empty())
	{
		set_int_it = col_has_val.begin();
		int i = *set_int_it;
		col_has_val.erase(set_int_it);

		for(set_int_it=link_from[i].begin(); set_int_it!=link_from[i].end(); set_int_it++)
		{
			//  for any k do A[j，k]：＝A[j，k]+A[i，k]；
			int j = *set_int_it;
			if(j==i) continue;

			for(temp_set_int_it=link_to[i].begin(); temp_set_int_it!=link_to[i].end(); temp_set_int_it++)
			{
				int k = *temp_set_int_it;

				if(k>i) col_has_val.insert(k);

				link_from[k].insert(j);

				int_to_set_int_it = link_to.find(j);
				if(int_to_set_int_it == link_to.end())
				{
					set<int> temp_s;
					temp_s.clear();
					link_to[j] = temp_s;
				}

				link_to[j].insert(k);
			}
		}
	}

	for(int_to_set_int_it = link_to.begin(); int_to_set_int_it != link_to.end(); int_to_set_int_it++)
	{
		int t_from = int_to_set_int_it->first;
		for(set_int_it = link_to[t_from].begin(); set_int_it != link_to[t_from].end(); set_int_it++)
		{
			int t_to = *set_int_it;
			if(t_to==t_from) continue;

			gen_sym_to_prod_it = gen_sym_to_prod_head.find(t_to);
			if(gen_sym_to_prod_it != gen_sym_to_prod_head.end())
			{
				ProdList* prod_list_cur = gen_sym_to_prod_head[t_to];
				while(prod_list_cur!=NULL)
				{
					Production* cur_prod = prod_list_cur->prod_pr;

				    sym_to_gen_ele_it = sym_to_gen_ele.find(t_from);
				    if(sym_to_gen_ele_it==sym_to_gen_ele.end())
				    {
					    set<vector<Element> > vec_set;
					    vec_set.clear();
					    sym_to_gen_ele[t_from] = vec_set;
				    }

					ele_set_it = sym_to_gen_ele[t_from].find(cur_prod->ele_vec);
					if(ele_set_it==sym_to_gen_ele[t_from].end())  // new production from unit prods
					{
						prod_list_end->next = new ProdList;
						prod_list_end = prod_list_end->next;
						prod_list_end->next = NULL;

						Production* new_prod = new Production;
						new_prod->genSym = t_from;

						vector<Element> new_ele_vec;
						new_ele_vec.clear();
						new_ele_vec.reserve(5);
						int ele_vec_size = cur_prod->ele_vec.size();
						for(int i=0; i<ele_vec_size; i++)
						{
							new_ele_vec.push_back(cur_prod->ele_vec[i]);
						}
						new_prod->ele_vec = new_ele_vec;

						prod_list_end->prod_pr = new_prod;

						sym_to_gen_ele[t_from].insert(new_ele_vec);

						gen_sym_to_prod_it = gen_sym_to_prod_head.find(t_from);
						if(gen_sym_to_prod_it==gen_sym_to_prod_head.end())
						{
							gen_sym_to_prod_head[t_from] = new ProdList;
							gen_sym_to_prod_end[t_from] = gen_sym_to_prod_head[t_from];
							gen_sym_to_prod_end[t_from]->next = NULL;
							gen_sym_to_prod_end[t_from]->prod_pr = new_prod;
						}
						else
						{
							gen_sym_to_prod_end[t_from]->next = new ProdList;
							gen_sym_to_prod_end[t_from] = gen_sym_to_prod_end[t_from]->next;
							gen_sym_to_prod_end[t_from]->next = NULL;
							gen_sym_to_prod_end[t_from]->prod_pr = new_prod;
						}

						if((ele_vec_size==1)&&(new_ele_vec[0].type==2))  // a char value
						{
							generating_sym.insert(t_from);
						}
					}

					prod_list_cur = prod_list_cur->next;
				}
			}
		}
	}

}

void CFG::removeUselessSym()
{
	set<int>::iterator set_int_it;

	/// --- first compute the reachable symbols, then the generating symbols
	reachable_sym.insert(start_sym);
	set<int> need_check;
	need_check.clear();
	need_check.insert(start_sym);
	while(!need_check.empty())
	{
		set_int_it = need_check.begin();
		int cur_sym = *set_int_it;
		need_check.erase(set_int_it);

		gen_sym_to_prod_it = gen_sym_to_prod_head.find(cur_sym);
		if(gen_sym_to_prod_it!=gen_sym_to_prod_head.end())
		{
			ProdList* cur_prod_list = gen_sym_to_prod_it->second;
			while(cur_prod_list!=NULL)
			{
				Production* cur_prod = cur_prod_list->prod_pr;
				int ele_vec_size = cur_prod->ele_vec.size();
				for(int i=0; i<ele_vec_size; i++)
				{
					int temp_type = cur_prod->ele_vec[i].type;
					if(temp_type==1)  // find a reachable symbol
					{
						int temp_sym = cur_prod->ele_vec[i].value;
						set_int_it = reachable_sym.find(temp_sym);
						if(set_int_it==reachable_sym.end())
						{
							reachable_sym.insert(temp_sym);
							need_check.insert(temp_sym);
						}
					}
					else   // find a reachable char value
					{
						int temp_terminal = cur_prod->ele_vec[i].value;
						reachable_terminal.insert(temp_terminal);
					}
				}

				cur_prod_list = cur_prod_list->next;
			}
		}
	}

	need_check.clear();
	set_int_it = need_check.begin();
	set_difference(reachable_sym.begin(),reachable_sym.end(),generating_sym.begin(),
			generating_sym.end(),inserter(need_check,set_int_it));

	bool has_changed = true;
	while(has_changed)
	{
		has_changed = false;

		for(set_int_it = need_check.begin(); set_int_it != need_check.end(); set_int_it++)
		{
			int cur_gen_sym = *set_int_it;

			gen_sym_to_prod_it = gen_sym_to_prod_head.find(cur_gen_sym);
			if(gen_sym_to_prod_it!=gen_sym_to_prod_head.end())
			{
				ProdList* cur_prod_list = gen_sym_to_prod_it->second;
				while(cur_prod_list!=NULL)
				{
					Production* cur_prod = cur_prod_list->prod_pr;
					int ele_vec_size = cur_prod->ele_vec.size();
					bool has_counter_support = false;

					for(int i=0; i<ele_vec_size; i++)
					{
						int temp_type = cur_prod->ele_vec[i].type;
						if(temp_type==1)
						{
							int temp_sym = cur_prod->ele_vec[i].value;
							set<int>::iterator temp_it;
							temp_it = generating_sym.find(temp_sym);
							if(temp_it==generating_sym.end())
							{
								has_counter_support = true;
								break;
							}
						}
					}

					if(!has_counter_support) // find a new generating symbol
					{
						generating_sym.insert(cur_gen_sym);
						has_changed = true;
						need_check.erase(set_int_it);
						break;
					}

					cur_prod_list = cur_prod_list->next;
				}
			}
		}
	}

	set_int_it = useful_sym.begin();
	set_intersection(reachable_sym.begin(),reachable_sym.end(),generating_sym.begin(),
			generating_sym.end(),inserter(useful_sym,set_int_it));
}

void CFG::print()
{
	cout << endl << "The CFG contains following informations:" << endl;
	cout << "The start symbol is " << sym_name[start_sym] << endl;

	ProdList* prod_list_cur = prod_list_head;
	if(prod_list_head->prod_pr!=NULL)
	{
		while(prod_list_cur!=NULL)
		{
			Production* cur_prod = prod_list_cur->prod_pr;
			cout << sym_name[cur_prod->genSym] << " := ";

			vector<Element> cur_ele_vec = cur_prod->ele_vec;
			if(cur_ele_vec.empty()) cout << "empson" << endl;
			else
			{
				for(ele_it = cur_ele_vec.begin(); ele_it != cur_ele_vec.end(); ele_it++)
				{
					Element cur_ele = *ele_it;
					if(cur_ele.type==1)  // dispose a symbol
					{
						cout << sym_name[cur_ele.value] << " ";
					}
					else   // dispose a char value
					{
						cout << "\"" << terminal_name [cur_ele.value] << "\"" << " ";
					}
				}

				cout << endl;
			}

			prod_list_cur = prod_list_cur->next;
		}
	}

	cout << endl;

}

void CFG::makeCNF()
{
	removeEmpson();
    removeUnitProduction();
    removeUselessSym();

    bool* is_sym_useful = new bool[sym_count+1];
    for(int i=1; i<=sym_count; i++)
    {
    	is_sym_useful[i] = false;
    }
    set<int>::iterator set_int_it;
    for(set_int_it = useful_sym.begin(); set_int_it != useful_sym.end(); set_int_it++)
    {
    	is_sym_useful[*set_int_it] = true;
    }

    string sym_prefix = "new_sym_";

    int* char_to_new_sym = new int[terminal_count+1];
    bool* is_char_useful = new bool[terminal_count+1];
    for(set_int_it = reachable_terminal.begin();
    		set_int_it != reachable_terminal.end(); set_int_it++)
    {
    	sym_count++;
    	char_to_new_sym[*set_int_it] = sym_count;
    	is_char_useful[*set_int_it] = false;

    	ostringstream convert;
    	convert << sym_prefix << sym_count;
    	sym_name[sym_count] = convert.str();
    }

	ProdList* prod_list_cur = prod_list_head;
	while(prod_list_cur!=NULL)
	{
		Production* cur_prod = prod_list_cur->prod_pr;
		if(cur_prod==NULL)
		{
			prod_list_cur = prod_list_cur->next;
			continue;
		}

		int cur_gen_sym = cur_prod->genSym;
		if(is_sym_useful[cur_gen_sym])
		{
			vector<Element> cur_ele_vec = cur_prod->ele_vec;
			int cur_ele_vec_size = cur_ele_vec.size();

			bool is_prod_useful = true;
			for(int vec_i=0; vec_i<cur_ele_vec_size; vec_i++)
			{
				if(cur_ele_vec[vec_i].type==1)
				{
					int temp_sym = cur_ele_vec[vec_i].value;
					if(!is_sym_useful[temp_sym])
					{
						is_prod_useful = false;
						break;
					}
				}
			}

			if(is_prod_useful)  // find an useful production here
			{
				int shift = 0;
				while((cur_ele_vec_size-shift)>2)  // a production with more than 2 symbols
				{
					int first_sym;
					if(cur_ele_vec[shift].type==2)  // a char value
					{
						int cur_char = cur_ele_vec[shift].value;
						is_char_useful[cur_char] = true;
						first_sym = char_to_new_sym[cur_char];
					}
					else first_sym = cur_ele_vec[shift].value;

					sym_count++;
					ostringstream convert;
			    	convert << sym_prefix << sym_count;
			    	sym_name[sym_count] = convert.str();
					int second_sym = sym_count;

					CNFProduction* new_CNFProd =
							new CNFProduction(cur_gen_sym,first_sym,second_sym);
					CNFProdList* new_CNFProdList = new CNFProdList(new_CNFProd);

					if(cnf_prod_pr.end_pr==NULL)
					{
						cnf_prod_pr.head_pr = new_CNFProdList;
						cnf_prod_pr.end_pr = new_CNFProdList;
					}
					else
					{
						cnf_prod_pr.end_pr->next = new_CNFProdList;
						cnf_prod_pr.end_pr = cnf_prod_pr.end_pr->next;
					}

					cur_gen_sym = second_sym;
					shift++;
				}

				if((cur_ele_vec_size-shift)==2)  // a production with 2 symbols
				{
					int first_sym;
					if(cur_ele_vec[shift].type==2)  // a char value
					{
						int cur_char = cur_ele_vec[shift].value;
						is_char_useful[cur_char] = true;
						first_sym = char_to_new_sym[cur_char];
					}
					else first_sym = cur_ele_vec[shift].value;

					int second_sym;
					if(cur_ele_vec[shift+1].type==2)  // a char value
					{
						int cur_char = cur_ele_vec[shift+1].value;
						is_char_useful[cur_char] = true;
						second_sym = char_to_new_sym[cur_char];
					}
					else second_sym = cur_ele_vec[shift+1].value;

					CNFProduction* new_CNFProd =
							new CNFProduction(cur_gen_sym,first_sym,second_sym);
					CNFProdList* new_CNFProdList = new CNFProdList(new_CNFProd);

					if(cnf_prod_pr.end_pr==NULL)
					{
						cnf_prod_pr.head_pr = new_CNFProdList;
						cnf_prod_pr.end_pr = new_CNFProdList;
					}
					else
					{
						cnf_prod_pr.end_pr->next = new_CNFProdList;
						cnf_prod_pr.end_pr = cnf_prod_pr.end_pr->next;
					}
				}
				else  // a production with one char value
				{
					int cur_terminal = cur_ele_vec[shift].value;
					useful_terminal.insert(cur_terminal);
					int first_sym = cur_terminal;

					head_sym_to_terminal_set[cur_gen_sym].insert(first_sym);
				}
			}
		}

		prod_list_cur = prod_list_cur->next;
	}

    for(set_int_it = reachable_terminal.begin();
    		set_int_it != reachable_terminal.end(); set_int_it++)
    {
    	int cur_terminal = *set_int_it;
    	int cur_gen_sym = char_to_new_sym[cur_terminal];
    	if(is_char_useful[cur_terminal])
    	{
    		useful_terminal.insert(cur_terminal);

    		head_sym_to_terminal_set[cur_gen_sym].insert(cur_terminal);
    	}
    }

    for(int i=0; i<=sym_count; i++)
    {
    	left_child_sym_to_cnf_prod.push_back(CNFProdListPr());
    }
	terminal_to_head_sym_vec.resize(terminal_count+1);

	map<int,set<int> >::iterator int_to_set_int_it;
	for(int_to_set_int_it=head_sym_to_terminal_set.begin();
			int_to_set_int_it!=head_sym_to_terminal_set.end(); int_to_set_int_it++)
	{
		int cur_head_sym = int_to_set_int_it->first;
		sym_gen_from_char.insert(cur_head_sym);

		set<int>& cur_char_set = int_to_set_int_it->second;
		for(set_int_it=cur_char_set.begin(); set_int_it!=cur_char_set.end(); set_int_it++)
		{
			terminal_to_head_sym_vec[*set_int_it].push_back(cur_head_sym);
		}
	}

	CNFProdList* cur_CNFProdList = cnf_prod_pr.head_pr;
	while(cur_CNFProdList!=NULL)
	{
		CNFProduction* cur_cnf_prod_pr = cur_CNFProdList->CNFProd_pr;
		int from_first = cur_cnf_prod_pr->from_first;

		CNFProdListPr& temp_CNFProdListPr = left_child_sym_to_cnf_prod[from_first];
		if(temp_CNFProdListPr.end_pr==NULL)
		{
			temp_CNFProdListPr.end_pr = new CNFProdList(cur_cnf_prod_pr);
			temp_CNFProdListPr.head_pr = temp_CNFProdListPr.end_pr;
		}
		else
		{
			temp_CNFProdListPr.end_pr->next = new CNFProdList(cur_cnf_prod_pr);
			temp_CNFProdListPr.end_pr = temp_CNFProdListPr.end_pr->next;
		}

		cur_CNFProdList = cur_CNFProdList->next;
	}



	delete[] is_sym_useful;
	delete[] is_char_useful;
	delete[] char_to_new_sym;

	////------------  free unused memory ------------------------////
	for(gen_sym_to_prod_it = gen_sym_to_prod_head.begin();
			gen_sym_to_prod_it != gen_sym_to_prod_head.end(); gen_sym_to_prod_it++)
	{
		ProdList* t_prod_list = gen_sym_to_prod_it->second;
		while(t_prod_list!=NULL)
		{
			ProdList* temp_prod_list = t_prod_list;
			t_prod_list = t_prod_list->next;
			delete temp_prod_list;
		}
	}

	prod_list_cur = prod_list_head;
	if(prod_list_head->prod_pr!=NULL)
	{
		while(prod_list_cur!=NULL)
		{
			Production* cur_prod = prod_list_cur->prod_pr;
			delete cur_prod;

			ProdList* temp_prod = prod_list_cur;
			prod_list_cur = prod_list_cur->next;
			delete temp_prod;
		}
	}
	else delete prod_list_head;

	prod_list_head = NULL;
}

void CFG::addSymPosition(int t_sym, Production* t_prod, int t_ele_pos)
{
	sym_to_pos_head_it = sym_to_pos_head.find(t_sym);
	if(sym_to_pos_head_it==sym_to_pos_head.end())
	{
		sym_to_pos_head[t_sym] = new SymPosList();
		sym_to_pos_end[t_sym] = sym_to_pos_head[t_sym];
		sym_to_pos_end[t_sym]->next = NULL;
		sym_to_pos_end[t_sym]->sym_pos_pr = new SymPosition();
		sym_to_pos_end[t_sym]->sym_pos_pr->prod = t_prod;
		sym_to_pos_end[t_sym]->sym_pos_pr->ele_pos = t_ele_pos;
	}
	else
	{
		sym_to_pos_end[t_sym]->next = new SymPosList();
		sym_to_pos_end[t_sym] = sym_to_pos_end[t_sym]->next;
		sym_to_pos_end[t_sym]->next = NULL;
		sym_to_pos_end[t_sym]->sym_pos_pr = new SymPosition();
		sym_to_pos_end[t_sym]->sym_pos_pr->prod = t_prod;
		sym_to_pos_end[t_sym]->sym_pos_pr->ele_pos = t_ele_pos;
	}
}

void CFG::printCNF()
{
	cout << endl << "The CNF contains following informations:" << endl;
	cout << "The start symbol is " << sym_name[start_sym] << endl;

	CNFProdList* cur_CNFProdList = cnf_prod_pr.head_pr;
	while(cur_CNFProdList!=NULL)
	{
		CNFProduction* cur_cnf_prod_pr = cur_CNFProdList->CNFProd_pr;
		int head_sym = cur_cnf_prod_pr->gen_sym;
		int from_first = cur_cnf_prod_pr->from_first;
		int from_second = cur_cnf_prod_pr->from_second;

		cout << sym_name[head_sym] << " := ";
		cout << sym_name[from_first] << " ";
		cout << sym_name[from_second] << endl;

		cur_CNFProdList = cur_CNFProdList->next;
	}

	map<int,set<int> >::iterator int_to_set_int_it;
	set<int>::iterator set_int_it;
	for(int_to_set_int_it=head_sym_to_terminal_set.begin();
			int_to_set_int_it!=head_sym_to_terminal_set.end(); int_to_set_int_it++)
	{
		int cur_head_sym = int_to_set_int_it->first;

		set<int>& cur_char_set = int_to_set_int_it->second;
		for(set_int_it=cur_char_set.begin(); set_int_it!=cur_char_set.end(); set_int_it++)
		{
			int cur_char = *set_int_it;
			cout << sym_name[cur_head_sym] << " := ";
			cout << "\"" << terminal_name[cur_char] << "\"" << endl;
		}
	}

	cout << endl;
}

CFG::~CFG()
{
	////------------  free unused memory ------------------------////
	for(int i=0; i<=sym_count; i++)
    {
    	CNFProdListPr& cur_CNFProdListPr = left_child_sym_to_cnf_prod[i];

    	CNFProdList* cur_CNFProdList = cur_CNFProdListPr.head_pr;
		while(cur_CNFProdList!=NULL)
		{
			CNFProdList* temp_CNFProdList = cur_CNFProdList;
			cur_CNFProdList = cur_CNFProdList->next;
			delete temp_CNFProdList;
		}
    }

	CNFProdList* cur_CNFProdList = cnf_prod_pr.head_pr;
	while(cur_CNFProdList!=NULL)
	{
		delete cur_CNFProdList->CNFProd_pr;

		CNFProdList* temp_CNFProdList = cur_CNFProdList;
		cur_CNFProdList = cur_CNFProdList->next;
		delete temp_CNFProdList;
	}
}


#endif /* CFG_H_ */
